<?php

namespace Laraautos\Dealer\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Laraautos\Dealer\Forms\Dealer as DealerForm;
use Laraautos\Dealer\Http\Requests\DealerRequest;
use Laraautos\Dealer\Interfaces\DealerRepositoryInterface;
use Laraautos\Dealer\Repositories\Eloquent\Filters\DealerResourceFilter;
use Laraautos\Dealer\Repositories\Eloquent\Presenters\DealerListPresenter;

/**
 * Resource controller class for dealer.
 */
class DealerResourceController extends BaseController
{

    /**
     * Initialize dealer resource controller.
     *
     *
     * @return null
     */
    public function __construct(DealerRepositoryInterface $dealer)
    {
        parent::__construct();
        $this->form = DealerForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('laraautos.dealer.modules'), 'dealer', guard_url('dealer'));
        $this->repository = $dealer;
    }

    /**
     * Display a list of dealer.
     *
     * @return Response
     */
    public function index(DealerRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(DealerResourceFilter::class)
            ->setPresenter(DealerListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('dealer::dealer.names'))
            ->view('dealer::dealer.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display dealer.
     *
     * @param Request $request
     * @param Model   $dealer
     *
     * @return Response
     */
    public function show(DealerRequest $request, DealerRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('dealer::dealer.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('dealer::dealer.show')
            ->output();
    }

    /**
     * Show the form for creating a new dealer.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(DealerRequest $request, DealerRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('dealer::dealer.name'))
            ->view('dealer::dealer.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new dealer.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DealerRequest $request, DealerRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('dealer::dealer.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('dealer/dealer/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/dealer/dealer'))
                ->redirect();
        }

    }

    /**
     * Show dealer for editing.
     *
     * @param Request $request
     * @param Model   $dealer
     *
     * @return Response
     */
    public function edit(DealerRequest $request, DealerRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('dealer::dealer.name'))
            ->view('dealer::dealer.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the dealer.
     *
     * @param Request $request
     * @param Model   $dealer
     *
     * @return Response
     */
    public function update(DealerRequest $request, DealerRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('dealer::dealer.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('dealer/dealer/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('dealer/dealer/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the dealer.
     *
     * @param Model   $dealer
     *
     * @return Response
     */
    public function destroy(DealerRequest $request, DealerRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('dealer::dealer.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('dealer/dealer/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('dealer/dealer/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
